from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, Session

DATABASE_URL = "sqlite+aiosqlite:///db/database.db"

engine = create_async_engine(DATABASE_URL, echo=True)


class SyncSession(Session):
    def get_bind(self, mapper=None, clause=None, **kwargs):
        return engine.sync_engine


SessionLocal = sessionmaker(
    class_=AsyncSession, sync_session_class=SyncSession, expire_on_commit=False
)

Base = declarative_base()


async def init_db():
    async with engine.begin() as conn:
        await conn.run_sync(Base.metadata.create_all)



import os
import sys
import click
import uvicorn

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(ROOT_DIR)


@click.command()
@click.argument("action", type=click.STRING, required=False)
def main(action: str):
    if action == "run":
        uvicorn.run(
            app="server:app",
            host="127.0.0.1",
            port=5656,
            workers=1,
        )


if __name__ == "__main__":
    main()



from factory.database import Base
from sqlalchemy import Column, Integer, String


class User(Base):
    __tablename__ = "users"

    id = Column(Integer, primary_key=True, index=True)
    username = Column(String, unique=True, index=True)
    email = Column(String, unique=True, index=True)
    full_name = Column(String)



from pydantic import BaseModel


class UserCreate(BaseModel):
    username: str
    email: str
    full_name: str


class UserUpdate(BaseModel):
    username: str
    email: str
    full_name: str


class UserResponse(BaseModel):
    id: int
    username: str
    email: str
    full_name: str

    class Config:
        orm_mode = True



from fastapi import FastAPI, Depends, HTTPException
from sqlalchemy.ext.asyncio import AsyncSession
from factory.database import SessionLocal, init_db
from api.crud import create_user, get_user, get_users, update_user, delete_user
from user_schema.schemas import UserCreate, UserUpdate, UserResponse

app = FastAPI()


@app.on_event("startup")
async def on_startup():
    await init_db()


# Dependency to get the database session
async def get_db():
    async with SessionLocal() as session:
        yield session


@app.post("/users/", response_model=UserResponse)
async def create_user_view(
    user: UserCreate, db: AsyncSession = Depends(get_db)
):
    return await create_user(db=db, user=user)


@app.get("/users/{user_id}", response_model=UserResponse)
async def read_user(user_id: int, db: AsyncSession = Depends(get_db)):
    db_user = await get_user(db, user_id)
    if db_user is None:
        raise HTTPException(status_code=404, detail="User not found")
    return db_user


@app.get("/users/", response_model=list[UserResponse])
async def read_users(
    skip: int = 0, limit: int = 10, db: AsyncSession = Depends(get_db)
):
    users = await get_users(db, skip=skip, limit=limit)
    return users


@app.put("/users/{user_id}", response_model=UserResponse)
async def update_user_view(
    user_id: int, user: UserUpdate, db: AsyncSession = Depends(get_db)
):
    db_user = await update_user(db, user_id, user)
    if db_user is None:
        raise HTTPException(status_code=404, detail="User not found")
    return db_user


@app.delete("/users/{user_id}", response_model=UserResponse)
async def delete_user_view(user_id: int, db: AsyncSession = Depends(get_db)):
    db_user = await delete_user(db, user_id)
    if db_user is None:
        raise HTTPException(status_code=404, detail="User not found")
    return db_user



from db_model.models import User
from user_schema.schemas import UserCreate, UserUpdate
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.future import select


async def create_user(db: AsyncSession, user: UserCreate):
    db_user = User(**user.dict())
    db.add(db_user)
    await db.commit()
    await db.refresh(db_user)
    return db_user


async def get_user(db: AsyncSession, user_id: int):
    result = await db.execute(select(User).filter(User.id == user_id))
    return result.scalar_one_or_none()


async def get_users(db: AsyncSession, skip: int = 0, limit: int = 10):
    result = await db.execute(select(User).offset(skip).limit(limit))
    return result.scalars().all()


async def update_user(db: AsyncSession, user_id: int, user: UserUpdate):
    result = await db.execute(select(User).filter(User.id == user_id))
    db_user = result.scalar_one_or_none()
    if db_user:
        for key, value in user.dict().items():
            setattr(db_user, key, value)
        await db.commit()
        await db.refresh(db_user)
    return db_user


async def delete_user(db: AsyncSession, user_id: int):
    result = await db.execute(select(User).filter(User.id == user_id))
    db_user = result.scalar_one_or_none()
    if db_user:
        await db.delete(db_user)
        await db.commit()
    return db_user
